package eu.atosresearch.health.prescripciones.client;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.allen_sauer.gwt.log.client.Log;

import com.google.gwt.core.client.GWT;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.rpc.AsyncCallback;

import eu.atosresearch.health.prescripciones.shared.AlarmaBean;


public class ControladorAlarmas {
	
	private static ControladorAlarmas INSTANCE = new ControladorAlarmas();
	private final BackendServiceAsync backendService = GWT.create(BackendService.class);
	
	//Leídos de repositorio
	private List<AlarmaBean> alarmasCargadas;
	
	public List<AlarmaBean> getAlarmasCargadas() {
		return alarmasCargadas;
	}

	public void setAlarmasCargadas(List<AlarmaBean> alarmasCargadas) {
		this.alarmasCargadas = alarmasCargadas;
	}

	//SINGLETON
	private ControladorAlarmas() {
	}
 
	public static ControladorAlarmas getInstance() {
        return INSTANCE;
    }    
	
	

	//METHODS
    //Carga las alarmas almacenadas
    public void cargarAlarmasAlmacenadas() {
    	Log.info("Cargar alarmas almacenadas en el servidor");
		backendService.getAlarmas(Controlador.getInstance().getUsuarioSistema(), new AsyncCallback<List<AlarmaBean>>() {
					public void onFailure(Throwable caught) {
						// Show the RPC error message to the user
						Log.error("Error al solicitar las alarmas." + caught);
					}
					public void onSuccess(List<AlarmaBean> alarmasBean) {
						Log.info("Alarmas recibidas");
						
						boolean hayAlarmasNuevas = false;
						for (AlarmaBean alarmaBean : alarmasBean) {
							//printLogPauta(pautaBean);
							Log.debug("Alarma: " + alarmaBean.getId() + "; fecha: " + alarmaBean.getFecha() + "; Prioridad: " + alarmaBean.getPrioridad() + "; Desc: " + alarmaBean.getDescripcion());
							if (alarmasCargadas != null) {
								boolean estaAlarmaEsNueva = true;
								for (AlarmaBean alarmaAntigua : alarmasCargadas) {
									if (alarmaAntigua.getId().equalsIgnoreCase(alarmaBean.getId()))
										estaAlarmaEsNueva = false;
								}
								hayAlarmasNuevas = hayAlarmasNuevas || estaAlarmaEsNueva;
							} else {
								hayAlarmasNuevas = true;
							}
						}
						
						if (hayAlarmasNuevas) {
							Log.debug("Encontradas alarmas nuevas");
							alarmasCargadas = ordenarPorPrioridad(alarmasBean);
							Controlador.getInstance().actualizarWidgetAlarmas();
						} else {
							Log.debug("No se encontraron alarmas nuevas");
						}
					}
				});
    }
    
    
    public void startNotificadorAlarmas(int segundos) {
    	Log.info("Arrancar el polling de alarmas");
    	cargarAlarmasAlmacenadas();
    	Timer t = new Timer() {
    		public void run() {
    	        Log.info("PROCESO AUTOMÁTICO - Carga de alarmas");
    	        cargarAlarmasAlmacenadas();
    		}
    	};
    	Log.info("Polling de alarmas establecido cada " + segundos + " segundos");
    	t.scheduleRepeating(segundos * 1000);
    }
    
    
    public void almacenarAlarmaHistorico(AlarmaBean alarma, String queSeHace, String cuandoSeHace) {
    	Log.info("Solicita guardar en el registro histórico de alarmas la acción...");
    	backendService.almacenarAlarma(alarma, queSeHace, cuandoSeHace, new AsyncCallback<AlarmaBean>() {
			@Override
			public void onFailure(Throwable caught) {
				Log.error("Error al almacenar acción sobre la alarma en el histórico.");
			}
			@Override
			public void onSuccess(AlarmaBean result) {
				Log.debug("La acción sobre la alarma se almacenó en el histórico");
			}
    	});
    }
    
    
    public void eliminarAlarma(final AlarmaBean alarma) {
    	Log.debug("Solicitud de eliminación de alarma al histórico");
    	Date ahoramismo = new Date();
    	almacenarAlarmaHistorico(alarma, "Alarma asumida", DateTimeFormat.getFormat("dd-MM-yyyy HH:mm").format(ahoramismo));
    	Log.info("Solicita eliminar la alarma...");
    	backendService.deleteAlarma(alarma.getId(), new AsyncCallback<Boolean>() {
			@Override
			public void onFailure(Throwable caught) {
				Log.error("Error al eliminar la alarma " + alarma.getId() + ". " + caught.getMessage());
			}
			@Override
			public void onSuccess(Boolean result) {
				Log.debug("Alarma eliminada");
				Date ahoramismo = new Date();
		    	almacenarAlarmaHistorico(alarma, "Alarma eliminada", DateTimeFormat.getFormat("dd-MM-yyyy HH:mm").format(ahoramismo));
			}
    	});
    }
    
    
    private List<AlarmaBean> ordenarPorPrioridad(List<AlarmaBean> listaOrigen) {
    	List<AlarmaBean> listaDestino = new ArrayList<AlarmaBean>();
    	
    	for (AlarmaBean alarmaOrigen : listaOrigen) {
    		if (alarmaOrigen.getPrioridad().equalsIgnoreCase(AlarmaBean.PRIORIDADALTA))
    			listaDestino.add(alarmaOrigen);
    	}
    	for (AlarmaBean alarmaOrigen : listaOrigen) {
    		if (alarmaOrigen.getPrioridad().equalsIgnoreCase(AlarmaBean.PRIORIDADMEDIA))
    			listaDestino.add(alarmaOrigen);
    	}
    	for (AlarmaBean alarmaOrigen : listaOrigen) {
    		if (alarmaOrigen.getPrioridad().equalsIgnoreCase(AlarmaBean.PRIORIDADBAJA))
    			listaDestino.add(alarmaOrigen);
    	}
    	return listaDestino;
    }
	
    
    private void printLogAlarma(AlarmaBean alarmaBean) {
		System.out.println("         alarma id=" + alarmaBean.getId());
		System.out.println("         alarma patientId: " + alarmaBean.getPacienteId());
		System.out.println("         fecha: " + alarmaBean.getFecha());
		System.out.println("         prioridad: " + alarmaBean.getPrioridad());
		System.out.println("         descipción: " + alarmaBean.getDescripcion());
		System.out.println("         acción: " + alarmaBean.getQueSeHizo());
		System.out.println("         fecha acción: " + alarmaBean.getCuandoSeHizo());
	}
}
