package eu.atosresearch.health.prescripciones.client;

import java.util.List;

import com.allen_sauer.gwt.log.client.Log;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;

import eu.atosresearch.health.prescripciones.client.widgets.pautas.SeleccionPacienteDialogWidget;
import eu.atosresearch.health.prescripciones.shared.PacienteBean;
import eu.atosresearch.health.prescripciones.shared.PautaBean;
import eu.atosresearch.health.prescripciones.shared.DiasBean;
import eu.atosresearch.health.prescripciones.shared.DiaBean;
import eu.atosresearch.health.prescripciones.shared.TomaBean;
import eu.atosresearch.health.prescripciones.shared.MedicamentoBean;


public class ControladorPautas {
	
	private static ControladorPautas INSTANCE = new ControladorPautas();
	private final BackendServiceAsync backendService = GWT.create(BackendService.class);
	
	private PacienteBean pacienteActual;
	private PautaBean pautaActual;
	
	//Leídos de repositorio
	private List<PautaBean> pautasCargadas;
	private List<TomaBean> tomasCargadas;
	
	
	
	//SINGLETON
	private ControladorPautas() {
	}
 
	public static ControladorPautas getInstance() {
        return INSTANCE;
    }    
	
	//GETTERS AND SETTERS

	public PacienteBean getPacienteActual() {
		return pacienteActual;
	}

	public void setPacienteActual(PacienteBean pacienteActual) {
		for (PautaBean paub : pautasCargadas) {
			if (paub.getPacienteId().equalsIgnoreCase(pacienteActual.getId()))
				pautaActual = paub;
		}
		this.pacienteActual = pacienteActual;
	}	
	
	public List<PautaBean> getPautasCargadas() {
		return pautasCargadas;
	}

	public void setPautasCargadas(List<PautaBean> pautasCargadas) {
		this.pautasCargadas = pautasCargadas;
	}

	public List<TomaBean> getTomasCargadas() {
		return tomasCargadas;
	}

	public void setTomasCargadas(List<TomaBean> tomasCargadas) {
		this.tomasCargadas = tomasCargadas;
	}

	public PautaBean getPautaActual() {
		return pautaActual;
	}

	public void setPautaActual(PautaBean pautaActual) {
		setPacienteActual(Controlador.getInstance().getPacienteBean(pautaActual.getPacienteId()));
		this.pautaActual = pautaActual;
	}

	    
	//MÉTODOS


	//Workflow en la vista de pautas y tomas 
	public void cargarAplicacion() {
		Controlador.getInstance().dejarSoloLaCabecera();
    	Log.debug("Diálogo de selección de paciente...");
		SeleccionPacienteDialogWidget seleccionPacienteDialog = new SeleccionPacienteDialogWidget();
		seleccionPacienteDialog.center();
		seleccionPacienteDialog.show();
		//Rellena el atributo pacienteActual
		//Después redirige a pacienteSeleccionado()
	}
	
    public void pacienteSeleccionado() {
		Log.debug("Cargando visualización para el paciente " + pacienteActual.getId());
		//PacienteDelSistemaWidget pacienteDelSistemaWidget = new PacienteDelSistemaWidget(pacienteActual.getId());
		//Controlador.getInstance().getMainVerticalRootPanel().add(pacienteDelSistemaWidget);
		
		
	}
    
    
    //Obtiene una pauta
    private PautaBean obtenerPautaBean(String id) {
    	for(PautaBean pautaBean : pautasCargadas) {
    		if (pautaBean.getId().equalsIgnoreCase(id))
    			return pautaBean;
    	}	
    	return null;
    	
    }
    
    
    //Recarga pautas y tomas
    public void refrescarPautasYTomas() {
    	Log.info("Refrescando pautas....");
    	cargarPautasAlmacenadas();
    	Log.info("Refrescando tomas....");
    	cargarTomasAlmacenadas();
    	Log.info("Pautas y tomas refrescándose de forma asíncrona");
    }
    
    
    //Carga las pautas almacenadas
    public void cargarPautasAlmacenadas() {
		backendService.getPautas(Controlador.getInstance().getUsuarioSistema(), new AsyncCallback<List<PautaBean>>() {
					public void onFailure(Throwable caught) {
						// Show the RPC error message to the user
						Log.error("Error al solicitar las pautas." + caught);
					}
					public void onSuccess(List<PautaBean> pautasBean) {
						Log.info("Pautas recibidas");
						pautasCargadas = pautasBean;

						//To Do
						

						//Logs
						for (PautaBean pautaBean : pautasBean) {
							//printLogPauta(pautaBean);
							Log.debug("Pauta: " + pautaBean.getNombreFichero() + "; fecha: " + pautaBean.getFechaInicio() + " a las " + pautaBean.getHoraInicio());
						}
					}
				});
    }
    
    
    //Carga las tomas almacenadas
    public void cargarTomasAlmacenadas() {
		backendService.getTomas(Controlador.getInstance().getUsuarioSistema(), new AsyncCallback<List<TomaBean>>() {
					public void onFailure(Throwable caught) {
						// Show the RPC error message to the user
						Log.error("Error al solicitar las tomas." + caught);
					}
					public void onSuccess(List<TomaBean> tomasBean) {
						Log.info("Tomas recibidas");
						tomasCargadas = tomasBean;
						
						//To Do
						

						for (TomaBean tomaBean : tomasBean) {
							//printLogToma(tomaBean);
							Log.debug("Toma: " + tomaBean.getId() + "; fecha: " + tomaBean.getHora());
						}
					}
				});
    }
    
    
    
    private void printLogPauta(PautaBean pautaBean) {
    	System.out.println("Nombre fichero: " + pautaBean.getNombreFichero());
    	System.out.println("Id: " + pautaBean.getId());
    	System.out.println("PacienteId: " + pautaBean.getPacienteId());
    	System.out.println("Fecha inicio: " + pautaBean.getFechaInicio());
		System.out.println("Hora inicio: " + pautaBean.getHoraInicio());
		System.out.println("Días:");
		
		DiasBean diasBean = pautaBean.getDiasBean();
		for (DiaBean diaBean : diasBean.getDiaBean()) {
			System.out.println("   dia id=" + diaBean.getId());
			System.out.println("      tomas");
			for (TomaBean tomaBean : diaBean.getTomasBean().getTomaBean()) {
				System.out.println("         toma id=" + tomaBean.getId());
				System.out.println("            hora: " + tomaBean.getHora());
				System.out.println("            medicamentos");
				for (MedicamentoBean medicamentoBean : tomaBean.getMedicamentosBean().getMedicamentoBean()) {
					System.out.println("               medicamento id=" + medicamentoBean.getId());
					System.out.println("                  principio activo:" + medicamentoBean.getPpoActivo());
					System.out.println("                  ubicacion:" + medicamentoBean.getUbicacion());
					System.out.println("                  observaciones:" + medicamentoBean.getObservaciones());
					System.out.println("                  criticidad:" + medicamentoBean.getCriticidad());
					System.out.println("                  dosis:" + medicamentoBean.getDosis());
				}
			}
		}
    }
    
    private void printLogToma(TomaBean tomaBean) {
		System.out.println("         toma id=" + tomaBean.getId());
		System.out.println("            hora: " + tomaBean.getHora());
		System.out.println("            medicamentos");
		for (MedicamentoBean medicamentoBean : tomaBean.getMedicamentosBean().getMedicamentoBean()) {
			System.out.println("               medicamento id=" + medicamentoBean.getId());
			System.out.println("                  principio activo:" + medicamentoBean.getPpoActivo());
			System.out.println("                  ubicacion:" + medicamentoBean.getUbicacion());
			System.out.println("                  observaciones:" + medicamentoBean.getObservaciones());
			System.out.println("                  criticidad:" + medicamentoBean.getCriticidad());
			System.out.println("                  dosis:" + medicamentoBean.getDosis());
		}
	}

	
    

}
