package eu.atosresearch.health.prescripciones.client.widgets;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.PasswordTextBox;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.event.dom.client.KeyPressHandler;
import com.google.gwt.event.dom.client.KeyPressEvent;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.event.dom.client.KeyUpHandler;

import eu.atosresearch.health.prescripciones.client.BackendService;
import eu.atosresearch.health.prescripciones.client.BackendServiceAsync;
import eu.atosresearch.health.prescripciones.client.Controlador;
import eu.atosresearch.health.prescripciones.client.widgets.images.Recursos;

import com.allen_sauer.gwt.log.client.Log;

public class AutorizacionWidget extends Composite {

	private final BackendServiceAsync backendService = GWT.create(BackendService.class);
	private final Recursos recursos = GWT.create(Recursos.class);
	
	private Label labelError;
	private TextBox usuarioTextBox;
	private PasswordTextBox passwordTextBox;
	
	public AutorizacionWidget() {
		
		VerticalPanel verticalPanel = new VerticalPanel();
		verticalPanel.setStyleName("portadaAutorizacion");
		initWidget(verticalPanel);
		verticalPanel.setSize("100%", "100%");
		
		//Imagen de fondo
		Image portada = new Image(recursos.teleoperadoraPauta());
		verticalPanel.add(portada);
		
		Label lblIdentificacin = new Label("Identificación:");
		lblIdentificacin.setStyleName("labelTituloSeccion");
		verticalPanel.add(lblIdentificacin);
		
		Grid grid = new Grid(4, 2);
		verticalPanel.add(grid);
		
		Label lblUsuario = new Label("Usuario:");
		lblUsuario.setStyleName("labelNormal");
		grid.setWidget(1, 0, lblUsuario);
		
		usuarioTextBox = new TextBox();
		grid.setWidget(1, 1, usuarioTextBox);
		
		Label lblContrasea = new Label("Contraseña:");
		lblContrasea.setStyleName("labelNormal");
		grid.setWidget(2, 0, lblContrasea);
		
		passwordTextBox = new PasswordTextBox();
		passwordTextBox.addKeyUpHandler(new KeyUpHandler() {
			@Override
			public void onKeyUp(KeyUpEvent event) {
				if (event.getNativeKeyCode() == KeyCodes.KEY_ENTER)
					intentarAcceder(usuarioTextBox.getText(), passwordTextBox.getText());
			}
		});
		grid.setWidget(2, 1, passwordTextBox);
		
		Button btnAcceder = new Button("acceder...");
		btnAcceder.setStyleName("botonSeparado");
		btnAcceder.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				intentarAcceder(usuarioTextBox.getText(), passwordTextBox.getText());
			}
		});
		grid.setWidget(3, 1, btnAcceder);
		
		labelError = new Label(" ");
		labelError.setStyleName("labelError");
		verticalPanel.add(labelError);
	}

	protected void intentarAcceder(final String usuarioTextBox, String passwordTextBox) {
		backendService.accesoPermitido(usuarioTextBox, passwordTextBox,
				new AsyncCallback<Boolean>() {
					public void onFailure(Throwable caught) {
						// Show the RPC error message to the user
						labelError.setText("Error al solicitar el acceso.");
						Log.error("Error al solicitar el acceso." + caught);
					}

					public void onSuccess(Boolean accesoPermitido) {
						if (accesoPermitido) {
							//labelError.setText("Acceso permitido.");
							Log.info("Acceso a la aplicación");
							Controlador.getInstance().setUsuarioSistema(usuarioTextBox);
							Controlador.getInstance().cargarMenu();
						} else {
							labelError.setText("No puede acceder actualmente con el Usuario/Contraseña que ha introducido.");
							Log.info("Intento de acceso con usuario / password no válido");
						}
					}
				});
	}

}
