package eu.atosresearch.health.prescripciones.server.pacientes;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.apache.log4j.Logger;

import eu.atosresearch.health.prescripciones.server.authorization.Acceso;
import eu.atosresearch.health.prescripciones.shared.PacienteBean;

public class ConsultaPaciente {
	static Logger logger = Logger.getLogger(ConsultaPaciente.class);
	
	public static PacienteBean datosPaciente(String id) {

		PropertiesConfiguration pacientes = new PropertiesConfiguration();
		try {
			pacientes.load(Acceso.class.getClassLoader().getResourceAsStream("eu/atosresearch/health/prescripciones/server/pacientes/pacientes.properties"));
			logger.debug("Pacientes cargado correctamente.");
		} catch (ConfigurationException ex) {
		    logger.error("Exception cargando pacientes: ", ex);
		}
		
		PacienteBean pb = new PacienteBean();
		pb.setId((id));
		pb.setNombre(pacientes.getString(id + ".nombre"));
		pb.setDoctor(pacientes.getString(id + ".doctor"));
		pb.setEmail(pacientes.getString(id + ".email"));
		pb.setMovil(pacientes.getString(id + ".movil"));
		pb.setContactoNombre(pacientes.getString(id + ".contacto.nombre"));
		pb.setContactoEmail(pacientes.getString(id + ".contacto.email"));
		pb.setContactoMovil(pacientes.getString(id + ".contacto.movil"));
		
		Set<String> teleasistentesActuales = new HashSet<String>();
		String[] teleasistentesArray = pacientes.getStringArray(id + ".teleasistente");
		for (int i=0; i<teleasistentesArray.length; i++) 
			teleasistentesActuales.add(teleasistentesArray[i]);
		
		pb.setTeleasistentes(teleasistentesActuales);
		
		return pb;
	}
	
	public static List<PacienteBean> pacientes(String usuario) {
		List<PacienteBean> resultado = new ArrayList<PacienteBean>();

		List<PacienteBean> todosPacientes = pacientes();
		for (PacienteBean pb : todosPacientes) {
			//if (doctor.equalsIgnoreCase(pb.getDoctor())) {
			//	resultado.add(pb);
			//}
			if (pb.getTeleasistentes().contains(usuario)) 
				resultado.add(pb);
		}
		
		return resultado;
	}
	
	public static List<PacienteBean> pacientes() {
		List<PacienteBean> resultado = new ArrayList<PacienteBean>();
		PropertiesConfiguration pacientes = new PropertiesConfiguration();
		try {
			pacientes.load(Acceso.class.getClassLoader().getResourceAsStream("eu/atosresearch/health/prescripciones/server/pacientes/pacientes.properties"));
			logger.debug("Pacientes cargado correctamente.");
		} catch (ConfigurationException ex) {
		    logger.error("Exception cargando pacientes: ", ex);
		}
		
		Iterator<String> it = pacientes.getKeys();
		Set<String> idsPacientes = new HashSet<String>();
		while (it.hasNext()) {
			String claveActual = it.next();
			logger.debug("Leido datos paciente: " + claveActual);
			claveActual = claveActual.substring(0, claveActual.indexOf("."));
			logger.debug("Transformado datos paciente: " + claveActual);
			idsPacientes.add(claveActual);		
		}
		logger.info("Leídos " + idsPacientes.size() + " pacientes.");
		
		it = idsPacientes.iterator();
		while (it.hasNext()) {
			String pacienteActual = it.next();
			
			PacienteBean pb = new PacienteBean();
			pb.setId(pacienteActual);
			pb.setNombre(pacientes.getString(pacienteActual + ".nombre"));
			pb.setDoctor(pacientes.getString(pacienteActual + ".doctor"));
			pb.setEmail(pacientes.getString(pacienteActual + ".email"));
			pb.setMovil(pacientes.getString(pacienteActual + ".movil"));
			pb.setContactoNombre(pacientes.getString(pacienteActual + ".contacto.nombre"));
			pb.setContactoEmail(pacientes.getString(pacienteActual + ".contacto.email"));
			pb.setContactoMovil(pacientes.getString(pacienteActual + ".contacto.movil"));
			
			Set<String> teleasistentesActuales = new HashSet<String>();
			String[] teleasistentesArray = pacientes.getStringArray(pacienteActual + ".teleasistente");
			for (int i=0; i<teleasistentesArray.length; i++) 
				teleasistentesActuales.add(teleasistentesArray[i]);
			
			pb.setTeleasistentes(teleasistentesActuales);
			
			resultado.add(pb);
			logger.debug("Leido paciente: " + pacienteActual);			
		}
		
		logger.debug("Encontrados " + resultado.size());
		return resultado;
	}
	
	
	public static boolean isTeleasistente(String pacienteId, String teleasistente) {
		logger.debug("Es teleasistente " + teleasistente + " de " + pacienteId + "?");

		if (datosPaciente(pacienteId).getTeleasistentes().contains(teleasistente)) 
			return true;
		else
			return false;
	}
	
}
